# frozen_string_literal: true
using RubyNext;
require 'yaml'

module Katarina
  module Parser
    class << self
      def parse(path)
        path
          .then(&File.method(:read))
          .then { |_1| YAML.load(_1, symbolize_names: true) }
          .then(&method(:build_schemas))
      end

      private

      def build_schemas(doc)
        path_and_method_pairs(doc[:paths]).flat_map do |pair|
          (__m__ = pair) && (((__m__.respond_to?(:deconstruct_keys) && (((__m_hash__ = __m__.deconstruct_keys([:path, :http_method])) || true) && ((Hash === __m_hash__) || Kernel.raise(TypeError, "#deconstruct_keys must return Hash")))) && ((__m_hash__.key?(:path) && __m_hash__.key?(:http_method)) && (((path = __m_hash__[:path]) || true) && ((http_method = __m_hash__[:http_method]) || true)))) || Kernel.raise(NoMatchingPatternError, __m__.inspect))
          (__m__ = doc[:paths][path][http_method]) && (((__m__.respond_to?(:deconstruct_keys) && (((__m_hash__ = __m__.deconstruct_keys([:responses, :summary])) || true) && ((Hash === __m_hash__) || Kernel.raise(TypeError, "#deconstruct_keys must return Hash")))) && ((__m_hash__.key?(:responses) && __m_hash__.key?(:summary)) && (((responses = __m_hash__[:responses]) || true) && ((summary = __m_hash__[:summary]) || true)))) || Kernel.raise(NoMatchingPatternError, __m__.inspect))
          responses.keys.map do |response_code|
            (__m__ = responses[response_code][:content].values) && (((__m__.respond_to?(:deconstruct) && (((__m_arr__ = __m__.deconstruct) || true) && ((Array === __m_arr__) || Kernel.raise(TypeError, "#deconstruct must return Array")))) && ((1 == __m_arr__.size) && ((content = __m_arr__[0]) || true))) || Kernel.raise(NoMatchingPatternError, __m__.inspect))
            Schema.new(
              path,
              *summary.split(' ').then { |_1| [_1.first, _1.last[1..]] },
              http_method,
              response_code,
              content[:schema]
            )
          end
        end
      end

      def path_and_method_pairs(paths)
        paths.keys.flat_map do |path|
          paths[path].keys.map do |http_method|
            { path: path, http_method: http_method }
          end
        end
      end
    end
  end
end
